package com.utopiaindustries.hseobservationsapp.activities;

import android.content.Intent;
import android.os.Bundle;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.lifecycle.ViewModelProvider;

import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.apiservice.ApiService;
import com.utopiaindustries.hseobservationsapp.apiservice.ApiServiceFactory;
import com.utopiaindustries.hseobservationsapp.helper.Helper;
import com.utopiaindustries.hseobservationsapp.helper.Preference;
import com.utopiaindustries.hseobservationsapp.utils.ProgressDialogFragment;
import com.utopiaindustries.hseobservationsapp.viewmodels.LoginViewModel;

public class LoginActivity extends AppCompatActivity {

    EditText tfEmail, tfPassword;
    Button btnLogin;
    LoginViewModel loginViewModel;
    ApiService apiService;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_login);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        if (!Helper.isNetworkConnected(this)) {
            Toast.makeText(this, "No Internet Connection", Toast.LENGTH_LONG).show();
        }

        initializeLayout();

        btnLogin.setOnClickListener(v -> {
            if (isValidate()) {
                /*loginViewModel.isUserAuthenticated(tfEmail.getText().toString().trim(),
                        tfPassword.getText().toString(),
                        new String[]{"ROLE_UIM_QC_APP_ACCESS_YES"});*/

                Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Preference.setMyStringPref(Helper.project_file,Helper.logInUser,this,tfEmail.getText().toString());

                Intent intent = new Intent(this, DashboardActivity.class);
                startActivity(intent);
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                finish();
            }
        });
    }

    public void initializeLayout() {
        tfEmail = findViewById(R.id.tf_email);
        tfPassword = findViewById(R.id.tf_password);
        btnLogin = findViewById(R.id.btn_login);
        apiService = ApiServiceFactory.getApiService();

        loginViewModel = new ViewModelProvider(this).get(LoginViewModel.class);

        loginViewModel.getLoadingState().observe(this, isLoading -> {
            if (isLoading != null && isLoading) {
                showProgressDialog();
            } else {
                dismissProgressDialog();
            }
        });

        loginViewModel.getErrorMessage().observe(this, errorResponse -> {
            if (errorResponse.isEmpty()) {
                Toast.makeText(this, "Something went wrong", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, errorResponse, Toast.LENGTH_SHORT).show();
            }

        });

        loginViewModel.getLoginUser().observe(this, loginUser -> {
            if (loginUser) {
                Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Preference.setMyStringPref(Helper.project_file,Helper.logInUser,this,tfEmail.getText().toString());

                Intent intent = new Intent(this, DashboardActivity.class);
                startActivity(intent);
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                finish();
            }
            else {
                Toast.makeText(this, "Login Failed", Toast.LENGTH_SHORT).show();
            }
        });

        /*loginViewModel.getUserLiveData().observe(this, user -> {
            if (user != null) {
                Preference.setMyBooleanPref(Helper.project_file, "isLoggedIn", getApplicationContext(), true);
                Helper.setPreferenceObject(getApplicationContext(), user, "DriverResponse");

                //Toast.makeText(this, "Welcome " + user.getTruckerName(), Toast.LENGTH_SHORT).show();
                Intent intent = new Intent(this, HomeActivity.class);
                startActivity(intent);
                overridePendingTransition(R.anim.slide_in_right, R.anim.slide_out_left);
                finish();
            } else {
                Toast.makeText(this, "Login Failed", Toast.LENGTH_SHORT).show();
            }
        });*/
    }

    public boolean isValidate() {
        boolean returnValue = true;
        String message = "";

        if (tfPassword.getText().toString().isEmpty()) {
            message = "Please enter password.";
            returnValue = false;
        }

        /*if (!Helper.isValidEmail(tfEmail.getText().toString())) {
            message = "Please enter valid email.";
            returnValue = false;
        }*/

        if (tfEmail.getText().toString().isEmpty()) {
            message = "Please enter user name.";
            returnValue = false;
        }

        if (!returnValue) {
            Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
        }

        return returnValue;
    }

    public void showProgressDialog() {
        ProgressDialogFragment progressDialog = new ProgressDialogFragment();
        progressDialog.setCancelable(false);
        progressDialog.show(getSupportFragmentManager(), "progressDialog");
    }

    public void dismissProgressDialog() {
        ProgressDialogFragment progressDialog = (ProgressDialogFragment) getSupportFragmentManager().findFragmentByTag("progressDialog");
        if (progressDialog != null) {
            progressDialog.dismiss();
        }
    }
}