package com.utopiaindustries.hseobservationsapp.activities.PermitToWorkForms;

import android.Manifest;
import android.content.ActivityNotFoundException;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Matrix;
import android.media.ExifInterface;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.provider.MediaStore;
import android.view.View;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.Toast;

import androidx.activity.EdgeToEdge;
import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.FileProvider;
import androidx.core.graphics.Insets;
import androidx.core.view.ViewCompat;
import androidx.core.view.WindowInsetsCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.adapters.PTWImageAdapter;
import com.utopiaindustries.hseobservationsapp.models.DocumentTypeImageModel;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.function.Consumer;

import pub.devrel.easypermissions.AfterPermissionGranted;
import pub.devrel.easypermissions.AppSettingsDialog;
import pub.devrel.easypermissions.EasyPermissions;

public class PermitTwoActivity extends AppCompatActivity implements EasyPermissions.PermissionCallbacks,
        EasyPermissions.RationaleCallbacks{

    private CheckBox checkboxCompany, checkboxContractor;
    RecyclerView ptwRecyclerView;
    PTWImageAdapter imagePaperAdapter;
    String paperFilePath = "no_pic";
    ArrayList<DocumentTypeImageModel> PaperImageList = new ArrayList<>();
    String docTypeId = "";
    String docTypeTitle = "";
    private static final int CAMERA_REQUEST_PAPER = 101;
    ImageView imgUpload, imgBack;

    // Activity Result Launcher for Paper Camera
    private final ActivityResultLauncher<Intent> paperCameraLauncher =
            registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {

                if (result.getResultCode() == RESULT_OK) {

                    File imageFile = new File(paperFilePath);
                    if (imageFile.exists()) {

                        Bitmap originalBitmap = fixImageRotation(paperFilePath);

                        //Bitmap timestampedBitmap = addTimestampToImage(originalBitmap);

                        bitmapToByteArrayAsync(
                                originalBitmap,
                                100, // Target size in KB
                                imageBytes -> {
                                    this.runOnUiThread(() -> {

                                        DocumentTypeImageModel documentImage = new DocumentTypeImageModel(
                                                docTypeId,
                                                docTypeTitle,
                                                imageBytes
                                        );

                                        //Log.e("doc-image: ",""+ documentImage);

                                        int position = PaperImageList.size();
                                        PaperImageList.add(documentImage);
                                        imagePaperAdapter.notifyItemInserted(position);

                                    });
                                },
                                error -> {
                                    error.printStackTrace();
                                }
                        );

                    } else {
                        Toast.makeText(this, "Image file not found!", Toast.LENGTH_SHORT).show();
                    }
                }
                else if (result.getResultCode() == RESULT_CANCELED) {
                    Toast.makeText(this, "Camera capture failed!", Toast.LENGTH_SHORT).show();
                }
                else {
                    Toast.makeText(this, "Camera capture failed!", Toast.LENGTH_SHORT).show();
                }
            });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EdgeToEdge.enable(this);
        setContentView(R.layout.activity_permit_two);
        ViewCompat.setOnApplyWindowInsetsListener(findViewById(R.id.main), (v, insets) -> {
            Insets systemBars = insets.getInsets(WindowInsetsCompat.Type.systemBars());
            v.setPadding(systemBars.left, systemBars.top, systemBars.right, systemBars.bottom);
            return insets;
        });

        initializeLayouts();

        checkboxCompany.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (checkboxCompany.isChecked()) {
                    checkboxContractor.setChecked(false);
                }
            }
        });

        checkboxContractor.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (checkboxContractor.isChecked()) {
                    checkboxCompany.setChecked(false);
                }
            }
        });

        imgUpload.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                openPaperCamera();
            }
        });

        imgBack.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
    }

    private void initializeLayouts() {
        checkboxCompany = findViewById(R.id.checkbox_company);
        checkboxContractor = findViewById(R.id.checkbox_contractor);
        imgUpload = findViewById(R.id.img_upload);
        imgBack = findViewById(R.id.img_back);

        ptwRecyclerView = findViewById(R.id.ptwRecyclerView);

        imagePaperAdapter = new PTWImageAdapter(PaperImageList, this, "");
        ptwRecyclerView.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
        ptwRecyclerView.setAdapter(imagePaperAdapter);

    }

    @AfterPermissionGranted(CAMERA_REQUEST_PAPER)
    public void openPaperCamera() {
        if (hasCameraPermission()) {
            try {
                Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                File photoFile = null;
                try {
                    photoFile = createImageFile();
                } catch (IOException ex) {
                    // Error occurred while creating the File
                }

                if (photoFile != null) {

                    Uri photoURI = FileProvider.getUriForFile(this,
                            getPackageName(),
                            photoFile);
                    takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                    //startActivityForResult(takePictureIntent, CAMERA_REQUEST);
                    paperCameraLauncher.launch(takePictureIntent);
                }
            } catch (ActivityNotFoundException e) {
                Toast.makeText(this, "Camera app not found", Toast.LENGTH_SHORT).show();
            }
        } else {
            // Ask for one permission
            String[] perms = {};
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                perms = new String[]{Manifest.permission.READ_MEDIA_IMAGES, Manifest.permission.CAMERA};
            } else {
                perms = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA};
            }

            EasyPermissions.requestPermissions(this, getString(R.string.rationale_camera), CAMERA_REQUEST_PAPER, perms);
        }
    }

    /*@AfterPermissionGranted(CAMERA_REQUEST_ITEM)
    public void openItemCamera() {
        if (hasCameraPermission()) {
            try {
                Intent takePictureIntent = new Intent(MediaStore.ACTION_IMAGE_CAPTURE);
                File photoFile = null;
                try {
                    photoFile = createImageFile("Item");
                } catch (IOException ex) {
                    // Error occurred while creating the File
                }

                if (photoFile != null) {
                    Uri photoURI = FileProvider.getUriForFile(this,
                            "com.utopia.inventorymanagement",
                            photoFile);
                    takePictureIntent.putExtra(MediaStore.EXTRA_OUTPUT, photoURI);
                    //startActivityForResult(takePictureIntent, CAMERA_REQUEST);
                    itemCameraLauncher.launch(takePictureIntent);
                }
            } catch (ActivityNotFoundException e) {
                Toast.makeText(this, "Camera app not found", Toast.LENGTH_SHORT).show();
            }
        } else {
            // Ask for one permission
            String[] perms = {};
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
                perms = new String[]{Manifest.permission.READ_MEDIA_IMAGES, Manifest.permission.CAMERA};
            } else {
                perms = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA};
            }

            EasyPermissions.requestPermissions(this, getString(R.string.rationale_camera), CAMERA_REQUEST_ITEM, perms);
        }
    }*/

    @Override
    public void onRequestPermissionsResult(int requestCode,
                                           @NonNull String[] permissions,
                                           @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);

        // EasyPermissions handles the request result.
        EasyPermissions.onRequestPermissionsResult(requestCode, permissions, grantResults, this);
    }

    @Override
    public void onPermissionsGranted(int requestCode, @NonNull List<String> perms) {

    }

    @Override
    public void onPermissionsDenied(int requestCode, @NonNull List<String> perms) {
        if (EasyPermissions.somePermissionPermanentlyDenied(this, perms)) {
            new AppSettingsDialog.Builder(this).build().show();
        }
    }

    private boolean hasCameraPermission() {
        String[] perms = {};
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            perms = new String[]{Manifest.permission.READ_MEDIA_IMAGES, Manifest.permission.CAMERA};
        } else {
            perms = new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.WRITE_EXTERNAL_STORAGE, Manifest.permission.CAMERA};

        }

        //Log.e("perms: ",""+perms);
        return EasyPermissions.hasPermissions(this, perms);
    }

    private Bitmap fixImageRotation(String imagePath) {
        Bitmap bitmap = BitmapFactory.decodeFile(imagePath);
        try {
            ExifInterface exif = new ExifInterface(imagePath);
            int orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_UNDEFINED);

            int rotationAngle = 0;
            switch (orientation) {
                case ExifInterface.ORIENTATION_ROTATE_90:
                    rotationAngle = 90;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    rotationAngle = 180;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_270:
                    rotationAngle = 270;
                    break;
            }

            if (rotationAngle != 0) {
                Matrix matrix = new Matrix();
                matrix.postRotate(rotationAngle);
                return Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        return bitmap;
    }

    private File createImageFile() throws IOException {
        // Create an image file name
        String timeStamp = new SimpleDateFormat("yyyyMMdd_HHmmss").format(new Date());
        String imageFileName = "JPEG_" + timeStamp + "_";
        File storageDir = getExternalFilesDir(Environment.DIRECTORY_PICTURES);
        File image = File.createTempFile(
                imageFileName,   //prefix
                ".jpg",          //suffix
                storageDir       //directory
        );

        paperFilePath = image.getAbsolutePath();

        return image;
    }

    @Override
    public void onRationaleAccepted(int requestCode) {

    }

    @Override
    public void onRationaleDenied(int requestCode) {

    }

    public void bitmapToByteArrayAsync(
            Bitmap bitmap,
            int targetSizeInKB,
            Consumer<byte[]> onSuccess,
            Consumer<Exception> onError
    ) {
        new Thread(() -> {
            try {
                if (bitmap == null) {
                    throw new IllegalArgumentException("Bitmap cannot be null.");
                }

                int targetSizeInBytes = targetSizeInKB * 1024;
                ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
                int minQuality = 10;
                int maxQuality = 100;
                int quality = maxQuality;

                // Binary search for the best quality that meets the target size
                while (minQuality <= maxQuality) {
                    byteArrayOutputStream.reset();
                    bitmap.compress(Bitmap.CompressFormat.JPEG, quality, byteArrayOutputStream);

                    int byteSize = byteArrayOutputStream.size();
                    if (byteSize > targetSizeInBytes) {
                        maxQuality = quality - 1;
                    } else {
                        minQuality = quality + 1;
                    }
                    quality = (minQuality + maxQuality) / 2;
                }

                onSuccess.accept(byteArrayOutputStream.toByteArray());
            } catch (Exception e) {
                onError.accept(e);
            }
        }).start();
    }
}