package com.utopiaindustries.hseobservationsapp.adapters;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.ExifInterface;
import android.net.Uri;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.RecyclerView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;
import com.utopiaindustries.hseobservationsapp.R;
import com.utopiaindustries.hseobservationsapp.models.DocumentTypeImageModel;
import com.utopiaindustries.hseobservationsapp.utils.FullScreenImageDialog;


import java.io.IOException;
import java.util.List;

public class PTWImageAdapter extends RecyclerView.Adapter<PTWImageAdapter.ImageViewHolder> {

    List<DocumentTypeImageModel> imageList;
    private Context context;
    String classType;

    public PTWImageAdapter(List<DocumentTypeImageModel> imageList, Context context, String classType) {
        this.imageList = imageList;
        this.context = context;
        this.classType = classType;
    }

    @NonNull
    @Override
    public ImageViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_image, parent, false);
        return new ImageViewHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull ImageViewHolder holder, int position) {
        //holder.imageView.setImageResource(R.drawable.img_load);

        /*Glide.with(context)
                .load(imageList.get(position)) // Glide will handle the decoding and placeholder
                .placeholder(R.drawable.img_load)
                .apply(new RequestOptions().centerCrop())
                .into(holder.imageView);*/

        holder.docTypeTitle.setText(imageList.get(position).getDocumentTypeName());
        byte[] imageData = imageList.get(position).getImage();

        // Load the image using Glide
        Bitmap bitmap = BitmapFactory.decodeByteArray(imageData, 0, imageData.length);
        Bitmap fixedBitmap = rotateImageIfNeeded(bitmap, position);

        Glide.with(context)
                .load(fixedBitmap)  // Load the adjusted bitmap
                //.placeholder(R.drawable.baseline_downloading) // Show loading image
                //.error(R.drawable.baseline_error) // Optional: Show error image
                .diskCacheStrategy(DiskCacheStrategy.ALL)
                .apply(new RequestOptions().centerCrop())
                .into(holder.imageView);

        /*if (classType.equalsIgnoreCase("Review")) {
            holder.btnDelete.setVisibility(View.GONE);
        }
        else {
            holder.btnDelete.setVisibility(View.VISIBLE);
        }
        // Set a click listener for the delete button
        holder.btnDelete.setOnClickListener(v -> {
            imageList.remove(position);
            notifyItemRemoved(position);
            notifyItemRangeChanged(position, imageList.size());
            Toast.makeText(context, "Image deleted", Toast.LENGTH_SHORT).show();
        });*/

        holder.imageView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
               // if (classType.equalsIgnoreCase("Review")) {
                    FullScreenImageDialog dialog = new FullScreenImageDialog(fixedBitmap);
                    dialog.show(((AppCompatActivity) v.getContext()).getSupportFragmentManager(), "FullScreenDialog");
               // }
            }
        });
    }

    @Override
    public int getItemCount() {
        return imageList.size();
    }

    public class ImageViewHolder extends RecyclerView.ViewHolder {
        ImageView imageView;
        Button btnDelete;
        TextView docTypeTitle;

        public ImageViewHolder(@NonNull View itemView) {
            super(itemView);
            imageView = itemView.findViewById(R.id.imageView);
            btnDelete = itemView.findViewById(R.id.btnDelete);
            docTypeTitle = itemView.findViewById(R.id.docTypeTitle);
        }
    }

    // Function to rotate the image if needed based on EXIF orientation
    private Bitmap rotateImageIfNeeded(Bitmap bitmap, int position) {
        try {
            // You can store EXIF data or add the path here if necessary
            ExifInterface exif = new ExifInterface(Uri.parse(imageList.get(position).toString()).getPath());
            int orientation = exif.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL);
            int rotate = 0;

            switch (orientation) {
                case ExifInterface.ORIENTATION_ROTATE_90:
                    rotate = 90;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_180:
                    rotate = 180;
                    break;
                case ExifInterface.ORIENTATION_ROTATE_270:
                    rotate = 270;
                    break;
            }

            if (rotate != 0) {
                android.graphics.Matrix matrix = new android.graphics.Matrix();
                matrix.postRotate(rotate);
                bitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
            }
        } catch (IOException e) {
            Log.e("ImageAdapter", "Error rotating image: " + e.getMessage());
        }

        return bitmap;
    }
}
