package com.utopiaindustries.hseobservationsapp.viewmodels;

import androidx.annotation.NonNull;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;
import androidx.lifecycle.ViewModel;

import com.utopiaindustries.hseobservationsapp.apiservice.ApiService;
import com.utopiaindustries.hseobservationsapp.apiservice.ApiServiceFactory;
import com.utopiaindustries.hseobservationsapp.models.QualityControlResponse;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class LoginViewModel extends ViewModel {

    private MutableLiveData<QualityControlResponse> userLiveData;
    private MutableLiveData<Boolean> userLoginLiveData;
    private MutableLiveData<String> errorLiveData;
    private MutableLiveData<Boolean> isLoading;
    private ApiService apiService;

    public LoginViewModel() {
        apiService = ApiServiceFactory.getApiService();
        userLiveData = new MutableLiveData<>();
        userLoginLiveData = new MutableLiveData<>();
        errorLiveData = new MutableLiveData<>();
        isLoading = new MutableLiveData<>();
    }

    public LiveData<QualityControlResponse> getUserLiveData() {
        return userLiveData;
    }

    public LiveData<Boolean> getLoadingState() {

        return isLoading;
    }

    public LiveData<String> getErrorMessage() {
        return errorLiveData;
    }

    public void isUserAuthenticated(String username, String password, String[] roles) {
        isLoading.setValue(true);
        apiService.isUserAuthenticated(username, password, roles).enqueue(new Callback<Boolean>() {
                    @Override
                    public void onResponse(@NonNull Call<Boolean> call, @NonNull Response<Boolean> response) {
                        isLoading.setValue(false);
                        //Log.e("onResponse-1: ", "Successful: "+response);
                        if (response.isSuccessful() && response.body() != null) {
                            //Log.e("onResponse-2: ", "Successful: "+response);
                            userLoginLiveData.setValue(response.body());
                        } else {
                            userLoginLiveData.setValue(false);
                        }
                    }

                    @Override
                    public void onFailure(@NonNull Call<Boolean> call, @NonNull Throwable t) {
                        //Log.e("onResponse-2: ", "failed"+t.getMessage());
                        isLoading.setValue(false);
                        errorLiveData.setValue(t.getMessage());
                    }
                });
    }

    public LiveData<Boolean> getLoginUser() {
        return userLoginLiveData;
    }

    public LiveData<QualityControlResponse> getUser() {
        return userLiveData;
    }

    public LiveData<String> getError() {
        return errorLiveData;
    }
}
